/* FLENG compiler - toplevel

  Copyright (c) 2021, Felix L. Winkelmann
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. The name of the authors may not be used to endorse or promote products
     derived from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

:- dynamic([pdef/2, r_atom/2, r_data/1, module_name/1, module_init/1]).
:- dynamic([no_warnings/0, debug_info/0, entry_point/2, uses/1]).
:- dynamic([drop_sequence/3, literal/2, symbol_loc/2, exported/1]).
:- dynamic([pdef_name/2, module_info/3, resolve_cache/2]).
:- dynamic([thread_count/1, include_path/1, profiling/0]).

:- initialization(main).

:- if(current_prolog_flag(dialect, gprolog)).
% GNU
:- include('version.pl').
:- include('compiler.pl').
:- include('index.pl').
:- include('lfa.pl').
:- include('asm.pl').

main :-
    argument_list(Args),
    main(Args).
:- endif.

:- if(current_prolog_flag(dialect, swi)).
% SWI
:- include(fleng('version.pl')).
:- include(fleng('compiler.pl')).
:- include(fleng('index.pl')).
:- include(fleng('lfa.pl')).
:- include(fleng('asm.pl')).

format_to_atom(Out, Fstr, Args) :-
    format_to_codes(Fstr, Args, S),
    atom_codes(Out, S).

g_assign(Var, Val) :- nb_setval(Var, Val).
g_read(Var, Val) :- nb_getval(Var, Val).
:- endif.

:- if(current_prolog_flag(dialect, yap)).
% YAP
:- use_module(library(lists)).
:- use_module(library(charsio)).
:- include('version.pl').
:- include('compiler.pl').
:- include('index.pl').
:- include('lfa.pl').
:- include('asm.pl').

start :-
    set_prolog_flag(syntax_errors, error),
    set_prolog_flag(unknown, error),
    unix(argv(Args)),
    catch(main(Args), E, quit(E)).

subtract([], _, []).
subtract([Element|Residue], Set, Difference) :-
	memberchk(Element, Set), !,
	subtract(Residue, Set, Difference).
subtract([Element|Residue], Set, [Element|Difference]) :-
	subtract(Residue, Set, Difference).

numbervars(Term) :- numbervars(Term, 0, _).

format_to_atom(Out, Fstr, Args) :-
    format_to_chars(Fstr, Args, S),
    atom_codes(Out, S).

save_to_file(Name) :- save(Name), halt(0).

quit(E) :-
    print_message(error, E),
    halt(1).

g_assign(Var, Val) :- nb_setval(Var, Val).
g_read(Var, Val) :- nb_getval(Var, Val).
:- endif.
